@@ -48,6 +48,9 @@
 #' @param pro_covar Optional dataframe of data with 4 named columns ("time","trend","covariate","value"), representing: (1) time, (2) the trend
 #'   affected, (3) the covariate number for models with more than one covariate affecting each
 #'   trend, and (4) the value of the covariate
+#' @param offset a string argument representing the name of the offset variable to be included. The variable name is in
+#' the data frame passed in, e.g. "offset". This only works when the data shape is "long". All transformations (such as log transformed effort)
+#' to the offset must be done before passing in the data.
 #' @param z_bound Optional hard constraints for estimated factor loadings -- really only applies to model with 1 trend. Passed in as a 2-element vector representing the lower and upper bound, e.g. (0, 100) to constrain positive
 #' @param z_model Optional argument allowing for elements of Z to be constrained to be proportions (each time series modeled as a mixture of trends). Arguments can be "dfa" (default) or "proportion"
 #' @param trend_model Optional argument to change the model of the underlying latent trend. By default this is set to 'rw', where the trend
@@ -69,7 +72,6 @@
 #'   This prior is a half-Student t prior, with the first argument of gp_theta_prior being the degrees of freedom (nu),
 #'   and the second element being the standard deviation
 #' @param expansion_prior Defaults to FALSE, if TRUE uses the parameter expansion prior of Ghosh & Dunson 2009
-#' @param ... Any other arguments to pass to [rstan::sampling()].
 #' @param par_list A vector of parameter names of variables to be estimated by Stan. If NULL, this will default to
 #'   c("x", "Z", "sigma", "log_lik", "psi","xstar") for most models -- though if AR / MA, or Student-t models are used
 #'   additional parameters will be monitored. If you want to use diagnostic tools in rstan, including moment_matching,
@@ -77,6 +79,10 @@
 #'   of diagnostic functions -- but making the models a lot larger for storage. Finally, this argument may be a custom string
 #'   of parameters to monitor, e.g. c("x","sigma")
 #' @param verbose Whether to print iterations and information from Stan, defaults to FALSE.
+#' @param weights Optional name of "weights" argument in data frame. This is only implemented when data
+#'   are in long format. If not entered, defaults to weights = 1 for all observations. The implementation of weights
+#'   varies slightly by family: Gaussian family models use -log(w_i) in the dispersion formula
+#' @param ... Any other arguments to pass to [rstan::sampling()].
 #' @details Note that there is nothing restricting the loadings and trends from
 #'   being inverted (i.e. multiplied by `-1`) for a given chain. Therefore, if
 #'   you fit multiple chains, the package will attempt to determine which chains
@@ -145,6 +151,13 @@
 #' # example of Gaussian process model with wide format data
 #' s <- sim_dfa(num_trends = 1, num_years = 20, num_ts = 3)
 #' m <- fit_dfa(y = s$y_sim, iter = 50, chains = 1, trend_model = "gp", n_knots = 5)
+#'
+#' # example of long format data
+#' s <- sim_dfa(num_trends = 1, num_years = 20, num_ts = 3)
+#' obs <- c(s$y_sim[1, ], s$y_sim[2, ], s$y_sim[3, ])
+#' long <- data.frame("obs" = obs, "ts" = sort(rep(1:3, 20)),
+#' "time" = rep(1:20, 3), "offset" = rep(0.1,length(obs)))
+#' m <- fit_dfa(y = long, data_shape = "long", offset = "offset", iter = 50, chains = 1)
 #' }
 fit_dfa <- function(y = y,
                     num_trends = 1,
@@ -165,6 +178,7 @@ fit_dfa <- function(y = y,
                     data_shape = c("wide", "long"),
                     obs_covar = NULL,
                     pro_covar = NULL,
+                    offset = NULL,
                     z_bound = NULL,
                     z_model = c("dfa", "proportion"),
                     trend_model = c("rw", "bs","ps", "gp"),
@@ -173,6 +187,7 @@ fit_dfa <- function(y = y,
                     par_list = NULL,
                     family = "gaussian",
                     verbose = FALSE,
+                    weights = NULL,
                     gp_theta_prior = c(3, 1),
                     expansion_prior = FALSE,
                     ...) {
@@ -219,6 +234,11 @@ fit_dfa <- function(y = y,
     y$ts <- as.numeric(as.factor(y[["ts"]]))
     N <- max(y[["time"]])
     P <- max(y[["ts"]])
+    if (!is.null(weights)) {
+      if(weights %in% names(y) == FALSE) {
+        stop("Error: weight name is not found in long data frame")
+      }
+    }
   }
 
   if (data_shape[1] == "wide") {
@@ -313,6 +333,7 @@ fit_dfa <- function(y = y,
   }
   nVariances <- length(unique(varIndx))
 
+  weights_vec <- NULL
   # indices of positive values - Stan can't handle NAs
   if (data_shape[1] == "wide") {
     row_indx_pos <- matrix(rep(seq_len(P), N), P, N)[!is.na(y)]
@@ -322,6 +343,11 @@ fit_dfa <- function(y = y,
     col_indx_na <- matrix(sort(rep(seq_len(N), P)), P, N)[is.na(y)]
     n_na <- length(row_indx_na)
     y <- y[!is.na(y)]
+    weights_vec <- rep(1, length(y))
+    if(!is.null(offset)) {
+      stop("Error: if offset is included, data shape must be long")
+    }
+    offset_vec <- rep(0, n_pos) # this is a dummy vec, not used
   } else {
     y <- y[which(!is.na(y[["obs"]])), ]
     row_indx_pos <- y[["ts"]]
@@ -331,6 +357,20 @@ fit_dfa <- function(y = y,
     row_indx_na <- matrix(1, 1, 1)[is.na(runif(1))]
     col_indx_na <- matrix(1, 1, 1)[is.na(runif(1))]
     n_na <- length(row_indx_na)
+
+    if(!is.null(weights)) {
+      weights_vec <- y[[weights]]
+    } else {
+      weights_vec <- rep(1, nrow(y))
+    }
+
+    offset_vec <- rep(0, nrow(y))
+    if(!is.null(offset)) {
+      if(offset %in% names(y) == FALSE) {
+        stop("Error: offset not found in data frame. Please check spelling")
+      }
+      offset_vec = y[[offset]]
+    }
     y <- y[["obs"]]
   }
 
@@ -501,7 +541,9 @@ fit_dfa <- function(y = y,
     est_gamma_params = est_gamma_params,
     est_nb2_params = est_nb2_params,
     gp_theta_prior = gp_theta_prior,
-    use_expansion_prior = as.integer(expansion_prior)
+    use_expansion_prior = as.integer(expansion_prior),
+    offset = offset_vec,
+    weights_vec = weights_vec
   )
 
   if (is.null(par_list)) {
